RPL 
INCLUDE Atmos.h

**********************************************************************
* Altitude ONLY quantities

* STACK DIAGRAM
* ( %H -> %H %T %P %Rho %Nu %g)
NULLNAME H
::
  DUPDUP ( duplicate the altitude to save it for later and to ...)
  2GETLAM %* SWAP 2GETLAM %+ %/ ( ...convert it to geopotential altitude from geometrical altitude)


  ( Find the index of the constants arrays that identifies the base level of the air quantities for the passed altitude)
  8GETLAM ( get the altitudes list)
  2DUPSWAP ' %>= POSCOMP ( return the index of the first element of the array that is greater than the current altitude)
  DUP#0=IT #9+  ( if no element is greater than the passed altitude, then use the latest)
  #1- ( we want the base altitude, which is the element just before that found -> decrease the index by 1)
  DUP#0=IT #1+  ( if we have now ZERO after subtracting 1 in the line before, it measn that the passed height was below sea level; use the index from element 1)
  
  DUP4UNROLL ( put the index on top of the stack)
  NTHCOMPDROP ( get the appropriate base altitude from the list)
  %- ( compute the delta altitude)
  
  6GETLAM ( get the base temperatures list)
  3PICK NTHCOMPDROP ( get the appropriate base temperature from the list)

  7GETLAM ( get the temperature gradients list)
  4PICK NTHCOMPDROP ( get the appropriate temperature gradient from the list)
  
  5GETLAM ( get the pressure ratios list)
  5ROLL NTHCOMPDROP ( get the appropriate pressure base ratio from the list)

  OVER %0= ( check if the temperature gradient is equal to 0)
  ITE
  ::
    1GETLAM %CHS ( get the hydrostatic constant and change its sign)
    5PICK %* ( multiply for the delta altitude)
    4PICK CKDeltaT_NSD %- ( pick the base temperature and subtract the non standard day correction,
                          ( becausethe base temperature to be used to determine the pressure must
                          ( be always the original one)
    %/ ( divide by the original base temperature )
    %EXP ( do the exponentiation)
  ;
  ::
    3PICK CKDeltaT_NSD %-  ( pick the base temperature and subtract the non standard day correction,
                           ( becausethe base temperature to be used to determine the pressure must
                           ( be always the original one)
    DUP
    4PICK ( pick the temperature gradient)
    7PICK %* ( and multiply it by the delta altitude)
    %+ ( sum to the base temperature to obtain the actual temperature)
    %/ ( obtain the ratio between base temperature and actual temperature)
    
    1GETLAM ( get the hydrostatic constant)
    4PICK %/ ( divide by the temperature gradient)
    %^ ( elevate to this power)
  ;
  %* ( obtain the effective pressure ratio)
  DUP
  4GETLAM %* ( multiply for the sea level pressure top obtain the actual pressure in Pascal)
  
  5UNROLL ( put the pressure at the top of the stack)
  4UNROLL ( put the pressure ratio immediately below it)
  ROT  %* %+ ( compute the temperature)
  
  SWAPOVER ( swap temperature and pressure ratio, duplicating the first)
  % 288.15 %/ %/ ( compute the density ratio)
  3GETLAM %* ( compute the actual density)
  
  ROTSWAP ( order T, P, rho)

	 3PICK %3 %2 %/ %^ 13GETLAM ( A ) %* 4PICK 14GETLAM ( B ) %+ %/ ( Sutherlands's law )
*  3PICK 13GETLAM 2DUP %/ %3 %2 %/ %^ UNROT %20 %6 %* SWAPOVER %+ UNROT %+ %/ %* 12GETLAM %*  ( Sutherlands's law)
*  3PICK %9 %* %5 %/ DUP %3 %2 %/ %^ % 0.0226968E-6 %* % 0.22481 %/ % 0.092903 %/ SWAP % 198.72 %+ %/ ( Formula on http://aero.stanford.edu/StdAtm.html )
  OVER %/ ( Kinematic viscosity)
  
  2GETLAM ( Earth Radius)
  DUP 7PICK ( altitude ) %+
  %/
  %SQ_
		12GETLAM ( g0 ) %*
;
  
* STACK DIAGRAM
* ( %T -> %T #idx)
NULLNAME T2H_Mid
::
  ( Establish the index of the constants arrays that identifies the base level of the air quantities for the passed temperature )
  ::
    DUP 6GETLAM ( base temperatures list) FIVE NTHCOMPDROP %> case BINT1
    DUP 6GETLAM ( base temperatures list) FIVE NTHCOMPDROP %= case BINT5 
    DUP 6GETLAM ( base temperatures list) FOUR NTHCOMPDROP %>= case BINT4 
    DUP 6GETLAM ( base temperatures list) THREE NTHCOMPDROP %>= case BINT3 
    DUP 6GETLAM ( base temperatures list) SEVEN NTHCOMPDROP %>= case BINT6 
    BINT7
  ;
  T_COMMON_T2H  ( computes the altitude and then all the values)
;

* STACK DIAGRAM
* ( %T -> %T #idx)
NULLNAME T2H_Inf
::
  ( Establish the index of the constants arrays that identifies the base level of the air quantities for the passed temperature )
  ::
  DUP 6GETLAM ( base temperatures list) TWO NTHCOMPDROP %>= case BINT1
  DUP 6GETLAM ( base temperatures list) SEVEN NTHCOMPDROP %>= case BINT6
  BINT7
  ;
  T_COMMON_T2H  ( computes the altitude and then all the values)
;
* STACK DIAGRAM
* ( %T -> %T #idx)
NULLNAME T2H_Sup
::
  ( Establish the index of the constants arrays that identifies the base level of the air quantities for the passed temperature )
  ::
  DUP 6GETLAM ( base temperatures list) EIGHT NTHCOMPDROP %<= case BINT8
  DUP 6GETLAM ( base temperatures list) SEVEN NTHCOMPDROP %<= case BINT7
  DUP 6GETLAM ( base temperatures list) SIX NTHCOMPDROP %<= case BINT6
  BINT1
  ;
  T_COMMON_T2H  ( computes the altitude and then all the values)
;

*Compute the values using the temperature in level 2 and the index identifying the base altitude values in level 1
* STACK DIAGRAM
* ( %T #idx -> %H %T %P %Rho %Nu )
NULLNAME T_COMMON_T2H
::
  8GETLAM ( get the altitude list)
  OVER NTHCOMPDROP ( get the appropriate altitude from the list)

  7GETLAM ( get the temperature gradients list)
  3PICK NTHCOMPDROP ( get the appropriate temperature gradient from the list)
  
  6GETLAM ( get the base temperature list)
  4ROLL NTHCOMPDROP ( get the appropriate base temperature from the list)
  
  OVER %0= ( check if the temperature gradient is equal to 0)
  ITE
  ::
    2DROP SWAPDROP ( drop everything and use the base altitude for computation)
  ;
  ::
    4ROLLSWAP ( put temperature in level 2 and base temperature in level 1)
    %- ( subtract them)
    SWAP %/ ( divide by temperature gradient)
    %+ ( sum to the base temperature)
  ;

  HPot>Geom ( convert the just found altitude to geometric altitude)

  H ( call the H procedure to compute everything from the altitude)
;

* STACK DIAGRAM
* ( %P -> %H %T %P %Rho %Nu )
NULLNAME P2H
::
  ( calculate the altitude from the pressure )
  4GETLAM %/ ( divide the pressure by the pressure at sea level )
  
  ( Find the index of the constants arrays that identifies the base level of the air quantities for the passed pressure )
  5GETLAM ( get the pressure ratios list)
  2DUPSWAP ' %<= POSCOMP ( return the index of the first element of the array that is lower than the current pressure)
  DUP#0=IT #9+  ( if no element is greater than the passed altitude, then use the latest)
  #1- ( we want the base altitude, which is the element just before that found -> decrease the index by 1)
  DUP#0=IT #1+  ( if we have now ZERO after subtracting 1 in the line before, it means that the passed height was below sea level; use the index from element 1)

  DUP4UNROLL ( put the index on top of the stack)
  NTHCOMPDROP ( get the appropriate base pressure ratio from the list)
  %/ ( compute the quantity P/P0/Pratio )
  
  8GETLAM ( get the altitude list)
  3PICK NTHCOMPDROP ( get the appropriate altitude from the list)

  7GETLAM ( get the temperature gradients list)
  4PICK NTHCOMPDROP ( get the appropriate temperature gradient from the list)
  
  6GETLAM ( get the base temperature list)
  5ROLL NTHCOMPDROP ( get the appropriate base temperature from the list)
  
  OVER %0= ( check if the temperature gradient is equal to 0)
  ITE
  ::
    ROT ( put the altitude on level 1)
    4ROLL %LN ( get the quantity P/P0/Pratio on level 1 and makes its natural log)
    ROT %* ( multiply it for the base temperature )
    1GETLAM %/ ( get the hydrostatic constant and divide by it )
    %- ( subtract from the base altitude)
    SWAPDROP ( drop the unused temperature gradient)
  ;
  ::
    ROTSWAP ( put the base altitude on level 2)
    ROT ( get the temperature gradient on level 1)
    4ROLL ( get the quantity P/P0/Pratio on level 1)
    OVER ( dulplicate the temperature gradient on level 1)
    1GETLAM  %CHS ( get the hydrostatic constant and change its sign)
    %/ ( divide the temperature gradient on level 2 by it)
    %^ %1- ( elevate the quantity P/P0/Pratio to this power and subtract 1)
    SWAP %/ %* %+
  ;

  HPot>Geom ( convert the just found altitude to geometric altitude)
  
  H ( call the H procedure to compute everything from the altitude)
;

* STACK DIAGRAM
* ( %Rho -> %H %T %P %Rho %Nu )
NULLNAME Rho2H
::
  ( calculate the altitude from the density )
  
  3GETLAM %/ ( divide the density by the density at sea level )

  ( Find the index of the constants arrays that identifies the base level of the air quantities for the passed density )
  9GETLAM ( get the density ratios list)
  OVER ' %<= POSCOMP ( return the index of the first element of the array that is lower than the current density)
  DUP#0=IT #9+  ( if no element is greater than the passed altitude, then use the latest)
  #1- ( we want the base altitude, which is the element just before that found -> decrease the index by 1)
  DUP#0=IT #1+  ( if we have now ZERO after subtracting 1 in the line before, it means that the passed height was below sea level; use the index from element 1)

  SWAP ( put the index on top of the stack)
  5GETLAM
  3PICK NTHCOMPDROP ( get the appropriate base pressure ratio from the list)
  
  8GETLAM ( get the altitude list)
  4PICK NTHCOMPDROP ( get the appropriate base altitude from the list)

  7GETLAM ( get the temperature gradients list)
  5PICK NTHCOMPDROP ( get the appropriate temperature gradient from the list)
  
  6GETLAM ( get the base temperature list)
  6ROLL NTHCOMPDROP ( get the appropriate base temperature from the list)
  
  OVER %0= ( check if the temperature gradient is equal to 0)
  ITE
  ::
    4ROLL OVER %/ ( compute the ratio between the base pressure ratio and base temperature)
    6GETLAM ONE NTHCOMPDROP ( get sea level temperature)
    %*
    %1/
    5ROLL %* %LN
    %*
    1GETLAM %/ ( get the hydrostatic constant and divide by it)
    SWAPDROP %-
  ;
  ::
    1GETLAM  %CHS ( get the hydrostatic constant and change its sign)
    3PICK %/  ( -GMR/T_grad )
    DUP
    3PICKSWAP %^
    6ROLL %/
    6GETLAM ONE NTHCOMPDROP %/ ( get sea level temperature and divide by it )
    6ROLL %*
    SWAP %1- %1/ %^
    SWAP %-
    SWAP %/
    %+
  ;

  HPot>Geom ( convert the just found altitude to geometric altitude)

  H ( call the H procedure to compute everything from the altitude)
;

* STACK DIAGRAM
* ( %Nu -> %H %T %P %Rho %Nu )
NULLNAME Nu2H
::
  ( calculate the altitude from the density )

  ( Find the index of the constants arrays that identifies the base level of the air quantities for the passed density )
  15GETLAM ( get the kinematic densities list)
  OVER ' %>= POSCOMP ( return the index of the first element of the array that is lower than the current density)
  DUP#0=IT #9+  ( if no element is greater than the passed altitude, then use the latest)
  #1- ( we want the base altitude, which is the element just before that found -> decrease the index by 1)
  DUP#0=IT #1+  ( if we have now ZERO after subtracting 1 in the line before, it means that the passed height was below sea level; use the index from element 1)

  SWAP ( put the index on top of the stack)
  5GETLAM
  3PICK NTHCOMPDROP ( get the appropriate base pressure ratio from the list)

  8GETLAM ( get the altitude list)
  4PICK NTHCOMPDROP ( get the appropriate base altitude from the list)

  7GETLAM ( get the temperature gradients list)
  5PICK NTHCOMPDROP ( get the appropriate temperature gradient from the list)

  6GETLAM ( get the base temperature list)
  6ROLL NTHCOMPDROP ( get the appropriate base temperature from the list)

  OVER %0= ( check if the temperature gradient is equal to 0)
  ITE
  ::
    4ROLL OVER %5 %2 %/ %^ 13GETLAM ( A ) %* %/ ( compute the ratio between the base pressure ratio and base temperature raised to 5/2 and multiplied by A)
    6GETLAM ONE NTHCOMPDROP %* ( multiply by sea level temperature )
    3GETLAM ( Rho0) %*  ( multiply by sea level density )
				OVER ( pick the base temperature) 14GETLAM ( B ) %+ ( and add B to it ) %* ( then multiply )
				5ROLL %* ( multiply by the kinematic viscosity inputted )
				%1/ %LN ( invert and make the logarithm )
    %* ( multiply by the base temperature )
    1GETLAM %/ ( get the hydrostatic constant and divide by it )
    SWAPDROP %- ( subtract the computed quantity from the base temperature )
  ;
  ::
				( eq to be solved numerically )
				1GETLAM  %CHS ( get the hydrostatic constant and change its sign)
    3PICK %/  ( -GMR/T_grad )
    DUP
    3PICKSWAP %^ ( base temperature raised to -GMR/T_grad )
    13GETLAM ( A ) %* ( multiply by A )
    7ROLL %/ ( divide by the inputted kinematic viscosity )
    6ROLL %/ ( divide by the base pressure ratio )
    6GETLAM ONE NTHCOMPDROP %/ ( get sea level temperature and divide by it )
    3GETLAM ( Rho0 ) %/ ( divide by the sea level density )
    ROT ( put the base temperature on level one )
    4ROLL ( put the base temperature gradient on level one )
    5PICK { LAM DH } BIND ( Store the base altitude in LAM DH; useful to be used as first guess )
				( From now on the GETLAM must be increased by 2 )
    ' LAM DH x* x+
    DUP 16GETLAM ( B ) x+
    SWAP
    4ROLL ( -GMR/T_grad ) %CHS %5 %2 %/ %+ x^
    x/
    x=
    FALSE SWAP
    ' LAM DH
				LAM DH
				NUMSOLVE
    ABND ( LAM DeltaH)
    2DROP
				%+ ( sum it to the base altitude )
  ;

  HPot>Geom ( convert the just found altitude to geometric altitude)

  H ( call the H procedure to compute everything from the altitude)
;


* STACK DIAGRAM
* ( %g -> %H %T %P %Rho %Nu %g )
NULLNAME g2H
::
  ( calculate the altitude from the acceleration of gravity )
		12GETLAM ( g0 ) %/ %1/ %SQRT %1- 2GETLAM ( Earth radius ) %*

  H ( call the H procedure to compute everything from the altitude)
;



*Convert Potential altitude to geometric altitude
* STACK DIAGRAM
* ( %H_pot -> %H_geom)
NULLNAME HPot>Geom
::
  2GETLAM ( Earth radius )
  2DUP %* UNROTSWAP %- %/
;
