RPL
INCLUDE Atmos.h

**********************************************************************
* Altitude + velocity quantities

* STACK DIAGRAM
* ( %H %VCAS -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
NULLNAME H_VCAS
::
		OVER H ( compute the atmosphere values for the passed height)
  3DROP UNROT2DROP ( preserves the pressure)
		SWAP ( Pressure on level 2, VCAS on level 1)
		( compute [qc/P0+1] from VCAS )
		VCAS2qc/P0+1
  %1-	4GETLAM ( P0) %* ( compute qc )
  
		%/ ( divide by the pressure ) %1/ %1+ ( and add 1 )

  qc/P+12Mach

  H_Mach ( call the H_Mach routine to do the actual computations)
;

*Computes the Impact pressure/Sea level pressure plus 1 (qc/P0+1) from VCAS
* STACK DIAGRAM
* (  %VCAS -> %qc/P0+1 )
NULLNAME VCAS2qc/P0+1
::
		( compute a0: sea level speed of sound )
		10GETLAM ( kappa ) 11GETLAM ( R-air) 6GETLAM ONE NTHCOMPDROP %* %* %SQRT
		2DUP %<= ITE :: ( if VCAS <= a0 )
		  %/ %SQ_
				10GETLAM ( kappa ) %1-	%*
				%2 %/
				%1+
				10GETLAM ( kappa )	DUP %1-	%/
				%^
		;
		:: ( else if if VCAS >= a0 )
		  %/ %SQ_
		  DUP
		  10GETLAM ( kappa ) DUP %1- %/ %2 %* %* %1- 10GETLAM ( kappa) %1- %1/ %^
		  SWAP
		  10GETLAM ( kappa) DUP %1- %/ %^
		  10GETLAM ( kappa ) %1+
				10GETLAM ( kappa ) %1-
				2DUP
				%/
				OVER %1/ %^
				ROT %2 %/
				10GETLAM ( kappa )
				4ROLL %/ %^
				%*
				%*
				%/
				%1/
		;

;

*Computes the Mach number from qc/P0+1 fro VCAS
* STACK DIAGRAM
* ( %qc/P+1 -> %Mach )
NULLNAME qc/P+12Mach
::
		DUP ( qc/P0+1 )
		( compute Mach number assuming it is subsonic )
		10GETLAM ( kappa ) DUP %1- %/ %1/ %^
		%1-
		%2 %* 10GETLAM ( kappa ) %1- %/ %SQRT

		DUP %1 %>= ITE :: ( The computed MAch number exceeds 1; it needs to be recomputed )
		  DROP ( The computed Mach number )
		  10GETLAM ( kappa ) %1+ %2 %/ ( store in LAM M2 the value [kappa+1]/2 )
				{ LAM M2 } BIND ( From now on the GETLAM must be increased by 2 )
				LAM M2 ' LAM M2 x*
				DUP
				LAM M2 %1/ 12GETLAM ( kappa) %* ' LAM M2 x*
				LAM M2 %2 %* 12GETLAM ( kappa) %1- SWAP %/
				x-
				x/
				12GETLAM ( kappa ) %1- %1/ x^
				x*
				x=
				FALSE SWAP
    ' LAM M2
				%1
				NUMSOLVE
    ABND ( LAM M2)
    2DROP
    %SQRT
		;
		::
		  SWAPDROP ( drop  qc/P0+1 )
		;
;

* STACK DIAGRAM
* ( %H %VEAS -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
NULLNAME H_VEAS
::
  OVER H ( compute the atmosphere values for the passed height)
  2DROP 4UNROLL 3DROP ( preserves the density)
  3GETLAM %/ %SQRT ( compute the ratio sqrt(Rho/Rho0)
  %/ ( compute the VTAS)

  H_VTAS
;

* STACK DIAGRAM
* ( %H %VTAS -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
NULLNAME H_VTAS
::
  OVER H ( compute the atmosphere values for the passed height)
  4DROP SWAPDROP ( preserves the temperature)
  10GETLAM 11GETLAM %* %* %SQRT ( compute the speed of sound )
  %/ ( compute the Mach number)

  H_Mach ( call the H_Mach routine to do the actual computations)
;

* STACK DIAGRAM
* ( %H %Mach -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
NULLNAME H_Mach
::
		SWAP
		H ( compute the atmosphere values for the passed height)
  DROP 6UNROLL ( put the kinematic viscosity on top: it will be used only for Reynolds)
		5ROLL ( put Mach number on level 1)

  SWAP 3GETLAM %/ %SQRT ( replace the density by the SQRT of the ratio density/density at sea level)
  
  OVER ( duplicate Mach)
  
  5ROLL 11GETLAM 10GETLAM  %* %* %SQRT ( sqrt[R_air*kappa_air*T] = a )
  %* ( this obtained is the VTAS)
  DUPROT %* ( V_EAS)
  SWAPROT ( Mach number in level 1, V_TAS in level 2, V_EAS in level 3)
  
  4ROLLOVER ( pressure on level 2, extra Mach number on level 1 )
  ( Compute the quantitiy [qc/P0 + 1] )
  Mach2qc/P

		%* ( multiply by the pressure )
		4GETLAM ( P0 ) %/ %1+ ( obtain qc/P0+1 )

  ( Compute the VCAS from the above found quantity )
  qc/P0+12VCAS

		4UNROLL ( order H V_CAS V_EAS V_TAS M)
 
  3PICK %2 %^ 3GETLAM %* %2 %/ ( compute the dynamic pressure)
  
  3PICK ( VTAS) 8ROLL ( kin visc) %/ ( Reynolds number per metre)
;

*Computes the quantity (q_c/P):
* Impact pressure divided by Sea level pressure plus 1
* STACK DIAGRAM
* ( %Mach -> %qc/P )
NULLNAME Mach2qc/P
::
  DUP %1 %<= ITE :: ( if subsonic )
		  10GETLAM ( kappa ) %1-
				DUP %2 %/
				ROT %SQ_ ( M ) %* %1+
				10GETLAM ( kappa ) ROT %/
				%^
				%1-
		;
		:: ( else if supersonic )
		  %SQ_ ( M )
				10GETLAM ( kappa ) %1+
		  DUPDUP
		  %2 %/
		  4PICK %* DUP
		  5ROLL 5ROLL %/ %2 %* 10GETLAM ( kappa ) %*
		  10GETLAM ( kappa ) %1-
		  5ROLL %/ %-
		  %/
		  10GETLAM ( kappa ) %1- %1/ %^
		  %* %1-
		;
;

*Computes VCAS from the quantity (q_c/P0 + 1):
* STACK DIAGRAM
* ( %qc/P0+1 -> %VCAS )
NULLNAME qc/P0+12VCAS
::
		( compute a0: sea level speed of sound )
		10GETLAM ( kappa ) 11GETLAM ( R-air) 6GETLAM ONE NTHCOMPDROP %* %* %SQRT
	 2DUP
		( compute VCAS assuming it is < a0 )
		%SQ_ %2 %* 10GETLAM ( kappa ) %1- %/
		SWAP
		10GETLAM ( kappa ) DUP %1- %/ %1/ %^
		%1- %* %SQRT
  DUP3PICK %> ITE :: ( if VCAS is instead greater than a0 )
		  DROP ( the VCAS computed is wrong )
				10GETLAM ( kappa ) DUP %1- %/ %2 %* ( store in LAM VCAS the value 2kappa/[kappa-1] )
				{ LAM VCAS } BIND ( From now on the GETLAM must be increased by 2 )
				' LAM VCAS
				SWAP
				x/
				DUP
				LAM VCAS x^
				SWAP
				%2 x^ LAM VCAS x* %1 x-
				12GETLAM ( kappa ) %1- %1/ x^
				x/
				SWAP ( q_c/P0 + 1 on level 1)
				12GETLAM ( kappa ) %1+
				12GETLAM ( kappa ) %1-
				2DUP
				%/
				OVER %1/ %^
				ROT %2 %/
				12GETLAM ( kappa )
				4ROLL %/ %^
				%*
				%/
				x=
				FALSE SWAP
    ' LAM VCAS
				% 400
				NUMSOLVE
    ABND ( LAM VCAS)
    2DROP
		;
	 :: ( VCAS as computed is ok because it is lower than a0 )
	   UNROT2DROP ( drop a0 and  qc/P0+1 )
		;
;


* STACK DIAGRAM
* ( %H %qEAS -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
NULLNAME H_qEAS
::
  3GETLAM %/ ( divide the dynamic pressure by Rho at sea level )
  %2 %* %SQRT ( obtain the equivalent air speed)

  H_VEAS ( call H_VEAS)
;

* STACK DIAGRAM
* ( %H %qTAS -> %H %VCAS %VEAS %VTAS %M %q %Re/m )
*NULLNAME H_qTAS
*::
*  OVER H ( compute the atmosphere values for the passed height)
*  2DROP 4UNROLL 3DROP ( preserves the density)
*  %/ ( divide the dynamic pressure by Rho just found )
*  %2 %* %SQRT ( obtain the true air speed)
*
*  H_VTAS ( call H_VTAS)
*;
